_LANG_CONFIG = {
  "mya": {
    "fleurs_lang_code": "my",
    "fleurs_country_code": "mm"
  },
  "ceb": {
    "fleurs_lang_code": "ceb",
    "fleurs_country_code": "ph"
  },
  "fil": {
    "fleurs_lang_code": "fil",
    "fleurs_country_code": "ph"
  },
  "ind": {
    "fleurs_lang_code": "id",
    "fleurs_country_code": "id"
  },
  "jav": {
    "fleurs_lang_code": "jv",
    "fleurs_country_code": "id"
  },
  "khm": {
    "fleurs_lang_code": "km",
    "fleurs_country_code": "kh"
  },
  "lao": {
    "fleurs_lang_code": "lo",
    "fleurs_country_code": "la"
  },
  "zlm": {
    "fleurs_lang_code": "ms",
    "fleurs_country_code": "my"
  },
  "tha": {
    "fleurs_lang_code": "th",
    "fleurs_country_code": "th"
  },
  "vie": {
    "fleurs_lang_code": "vi",
    "fleurs_country_code": "vn"
  }
}
