_LANG_CONFIG = {
  "ace": {
    "name": "Acehnese",
    "source_subset": "ace"
  },
  "ban": {
    "name": "Balinese",
    "source_subset": "ban"
  },
  "bcl": {
    "name": "Central Bicolano",
    "source_subset": "bcl"
  },
  "bjn": {
    "name": "Banjarese",
    "source_subset": "bjn"
  },
  "bug": {
    "name": "Buginese",
    "source_subset": "bug"
  },
  "cbk": {
    "name": "Chavacano",
    "source_subset": "cbk-zam"
  },
  "gor": {
    "name": "Gorontalo",
    "source_subset": "gor"
  },
  "ilo": {
    "name": "Ilokano",
    "source_subset": "ilo"
  },
  "ind": {
    "name": "Indonesian",
    "source_subset": "id"
  },
  "jav": {
    "name": "Javanese",
    "source_subset": "jv"
  },
  "khm": {
    "name": "Khmer",
    "source_subset": "km"
  },
  "lao": {
    "name": "Lao",
    "source_subset": "lo"
  },
  "mad": {
    "name": "Madurese",
    "source_subset": "mad"
  },
  "map_bms": {
    "name": "Banyumasan (Dialect of Javanese)",
    "source_subset": "map-bms"
  },
  "min": {
    "name": "Minangkabau",
    "source_subset": "min"
  },
  "mnw": {
    "name": "Mon",
    "source_subset": "min"
  },
  "mya": {
    "name": "Burmese",
    "source_subset": "my"
  },
  "nia": {
    "name": "Nias",
    "source_subset": "nia"
  },
  "pag": {
    "name": "Pangasinan",
    "source_subset": "pag"
  },
  "pam": {
    "name": "Kapampangan",
    "source_subset": "pam"
  },
  "shn": {
    "name": "Shan",
    "source_subset": "shn"
  },
  "sun": {
    "name": "Sundanese",
    "source_subset": "su"
  },
  "tet": {
    "name": "Tetum",
    "source_subset": "tet"
  },
  "tgl": {
    "name": "Tagalog",
    "source_subset": "tl"
  },
  "tha": {
    "name": "Thai",
    "source_subset": "th"
  },
  "vie": {
    "name": "Vietnamese",
    "source_subset": "vi"
  },
  "war": {
    "name": "Waray",
    "source_subset": "war"
  }
}
