_URLS_DICT = {
    "LIO": {
        "LIO_ave_maria_transcription":{
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_5fabcbda_6835_4ae3_b66f_21daae70d7b6/datastream/OBJ/download",
        },
        "LIO_positional_transcription":{
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_c1c1b5d6_d607_44c2_8f39_cda2041275aa/datastream/OBJ/download"
        },
        "LIO_frogstory_transcription":{
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_f290b09f_c351_46fe_8d86_d2ac624294c1/datastream/OBJ/download"
        },
        "LIO_frogstory_stan_jaya_transcription":{
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_1f24eace_b603_46f4_96d0_0054305caaf6/datastream/OBJ/download"
        },
        "LIO_history_transcription":{
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_b3b9b4de_a76e_440d_bf03_12c51ddc7954/datastream/OBJ/download"
        },
        "LIO_scorpion_story_transcription":{
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_d3b552ce_72d4_4d94_8510_8626b5cb91da/datastream/OBJ/download"
        },
        "LIO_our_father_transcription": {
             "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_50477a6a_12c6_4ae1_976a_9770f356062e/datastream/OBJ/download"
        },
        "LIO_credo_transcription": {
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_90f05ac6_813b_4570_b25c_f8d8b12bf8f0/datastream/OBJ/download"
        },
        "LIO_father_son_holy_spirit":{
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_9016339d_164f_464e_8653_9d941dd622b4/datastream/OBJ/download"
        },
        "LIO_glory_be_transcription": {
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_9a5f906c_bd51_40a3_a07a_39a99bb21d5a/datastream/OBJ/download"
        },
        "LIO_sambal_recipe_transcription": {
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_c2c00068_d01e_4ee3_877d_b42e1b0e77c5/datastream/OBJ/download"
        },
        "LIO_bridewealth_transcription": {
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_02e08440_721c_4a04_9ca5_7c9fb86eca2d/datastream/OBJ/download"
        },
        "LIO_gawi_kelimutu_transcription":{
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_d1e32c05_4e58_4171_9c59_0b394a13dc0b/datastream/OBJ/download"
        },
        "LIO_song_ine_transcription": {
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_d504e40b_e70e_4062_b406_8d69e162d392/datastream/OBJ/download"
        }
    },
    "ENDE": {
        "ENDE_positional_transcription": {
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_1655f827_74d6_47c9_83eb_c9be02a3ac3d/datastream/OBJ/download"
        },
        "ENDE_frogstory_transcription":{
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_b990239e_6229_4c58_96c3_71755040a8bb/datastream/OBJ/download"
        }
    },
    "NAGE":{
        "Nage_Ondorea_Barat_Lexirumah_Wordlist_Original":{
            "text_path": "https://archive.mpi.nl/tla/islandora/object/tla%3A1839_34f3575a_f83f_4674_a1ac_8abd0006e151/datastream/OBJ/download"
        }
    }
}